@@ -31,7 +31,6 @@
 package com.google.protobuf;
 
 import com.google.protobuf.Descriptors.FieldDescriptor;
-
 import java.io.IOException;
 import java.util.ArrayList;
 import java.util.List;
@@ -714,12 +713,14 @@ class MessageReflection {
   }
 
   /**
-   * Parses a single field into MergeTarget. The target can be Message.Builder,
-   * FieldSet or MutableMessage.
+   * Parses a single field into MergeTarget. The target can be Message.Builder, FieldSet or
+   * MutableMessage.
    *
-   * Package-private because it is used by GeneratedMessage.ExtendableMessage.
+   * <p>Package-private because it is used by GeneratedMessage.ExtendableMessage.
    *
    * @param tag The tag, which should have already been read.
+   * @param unknownFields If not null, unknown fields will be merged to this {@link
+   *     UnknownFieldSet}, otherwise unknown fields will be discarded.
    * @return {@code true} unless the tag is an end-group tag.
    */
   static boolean mergeFieldFrom(
@@ -728,7 +729,8 @@ class MessageReflection {
       ExtensionRegistryLite extensionRegistry,
       Descriptors.Descriptor type,
       MergeTarget target,
-      int tag) throws IOException {
+      int tag)
+      throws IOException {
     if (type.getOptions().getMessageSetWireFormat() &&
         tag == WireFormat.MESSAGE_SET_ITEM_TAG) {
       mergeMessageSetExtensionFromCodedStream(
@@ -792,7 +794,11 @@ class MessageReflection {
     }
 
     if (unknown) {  // Unknown field or wrong wire type.  Skip.
-      return unknownFields.mergeFieldFrom(tag, input);
+      if (unknownFields != null) {
+        return unknownFields.mergeFieldFrom(tag, input);
+      } else {
+        return input.skipField(tag);
+      }
     }
 
     if (packed) {
@@ -844,7 +850,9 @@ class MessageReflection {
             // If the number isn't recognized as a valid value for this enum,
             // drop it.
             if (value == null) {
-              unknownFields.mergeVarintField(fieldNumber, rawValue);
+              if (unknownFields != null) {
+                unknownFields.mergeVarintField(fieldNumber, rawValue);
+              }
               return true;
             }
           }
@@ -947,7 +955,7 @@ class MessageReflection {
         mergeMessageSetExtensionFromBytes(
             rawBytes, extension, extensionRegistry, target);
       } else { // We don't know how to parse this. Ignore it.
-        if (rawBytes != null) {
+        if (rawBytes != null && unknownFields != null) {
           unknownFields.mergeField(typeId, UnknownFieldSet.Field.newBuilder()
               .addLengthDelimited(rawBytes).build());
         }
