@@ -35,9 +35,6 @@
 // This file contains tests and benchmarks.
 
 #include <memory>
-#ifndef _SHARED_PTR_H
-#include <google/protobuf/stubs/shared_ptr.h>
-#endif
 #include <vector>
 
 #include <google/protobuf/io/coded_stream.h>
@@ -63,6 +60,7 @@ namespace protobuf {
 namespace io {
 namespace {
 
+
 // ===================================================================
 // Data-Driven Test Infrastructure
 
@@ -736,7 +734,7 @@ TEST_F(CodedStreamTest, ReadStringImpossiblyLargeFromStringOnStack) {
 }
 
 TEST_F(CodedStreamTest, ReadStringImpossiblyLargeFromStringOnHeap) {
-  google::protobuf::scoped_array<uint8> buffer(new uint8[8]);
+  std::unique_ptr<uint8[]> buffer(new uint8[8]);
   CodedInputStream coded_input(buffer.get(), 8);
   string str;
   EXPECT_FALSE(coded_input.ReadString(&str, 1 << 30));
@@ -1296,35 +1294,6 @@ void CodedStreamTest::SetupTotalBytesLimitWarningTest(
   *out_warnings = scoped_log.GetMessages(WARNING);
 }
 
-TEST_F(CodedStreamTest, TotalBytesLimitWarning) {
-  std::vector<string> errors;
-  std::vector<string> warnings;
-  SetupTotalBytesLimitWarningTest(10240, 1024, &errors, &warnings);
-
-  EXPECT_EQ(0, errors.size());
-
-  EXPECT_EQ(1, warnings.size());
-  EXPECT_PRED_FORMAT2(testing::IsSubstring,
-    "The total number of bytes read was 2048",
-    warnings[0]);
-}
-
-TEST_F(CodedStreamTest, TotalBytesLimitWarningDisabled) {
-  std::vector<string> errors;
-  std::vector<string> warnings;
-
-  // Test with -1
-  SetupTotalBytesLimitWarningTest(10240, -1, &errors, &warnings);
-  EXPECT_EQ(0, errors.size());
-  EXPECT_EQ(0, warnings.size());
-
-  // Test again with -2, expecting the same result
-  SetupTotalBytesLimitWarningTest(10240, -2, &errors, &warnings);
-  EXPECT_EQ(0, errors.size());
-  EXPECT_EQ(0, warnings.size());
-}
-
-
 TEST_F(CodedStreamTest, RecursionLimit) {
   ArrayInputStream input(buffer_, sizeof(buffer_));
   CodedInputStream coded_input(&input);
@@ -1425,9 +1394,6 @@ TEST_F(CodedStreamTest, InputOver2G) {
   EXPECT_EQ(0, errors.size());
 }
 
-// ===================================================================
-
-
 }  // namespace
 }  // namespace io
 }  // namespace protobuf
