@@ -37,9 +37,10 @@
 #include <map>
 #include <vector>
 #include <google/protobuf/stubs/common.h>
-
+#include <gmock/gmock.h>
 // Disable death tests if we use exceptions in CHECK().
-#if !PROTOBUF_USE_EXCEPTIONS && defined(GTEST_HAS_DEATH_TEST)
+#if !PROTOBUF_USE_EXCEPTIONS && defined(GTEST_HAS_DEATH_TEST) && \
+    !GTEST_OS_WINDOWS
 #define PROTOBUF_HAS_DEATH_TEST
 #endif
 
@@ -82,10 +83,10 @@ class ScopedMemoryLog {
   virtual ~ScopedMemoryLog();
 
   // Fetches all messages with the given severity level.
-  const vector<string>& GetMessages(LogLevel error);
+  const std::vector<string>& GetMessages(LogLevel error);
 
  private:
-  map<LogLevel, vector<string> > messages_;
+  std::map<LogLevel, std::vector<string> > messages_;
   LogHandler* old_handler_;
 
   static void HandleLog(LogLevel level, const char* filename, int line,
