@@ -365,10 +365,10 @@ TEST_F(DefaultFieldComparatorTest,
 
   // +inf should be equal even though they are not technically within margin or
   // fraction.
-  message_1_.set_optional_float(numeric_limits<float>::infinity());
-  message_2_.set_optional_float(numeric_limits<float>::infinity());
-  message_1_.set_optional_double(numeric_limits<double>::infinity());
-  message_2_.set_optional_double(numeric_limits<double>::infinity());
+  message_1_.set_optional_float(std::numeric_limits<float>::infinity());
+  message_2_.set_optional_float(std::numeric_limits<float>::infinity());
+  message_1_.set_optional_double(std::numeric_limits<double>::infinity());
+  message_2_.set_optional_double(std::numeric_limits<double>::infinity());
   comparator_.SetFractionAndMargin(field_float, 0.0, 0.0);
   comparator_.SetFractionAndMargin(field_double, 0.0, 0.0);
   EXPECT_EQ(FieldComparator::SAME,
@@ -380,10 +380,10 @@ TEST_F(DefaultFieldComparatorTest,
 
   // -inf should be equal even though they are not technically within margin or
   // fraction.
-  message_1_.set_optional_float(-numeric_limits<float>::infinity());
-  message_2_.set_optional_float(-numeric_limits<float>::infinity());
-  message_1_.set_optional_double(-numeric_limits<double>::infinity());
-  message_2_.set_optional_double(-numeric_limits<double>::infinity());
+  message_1_.set_optional_float(-std::numeric_limits<float>::infinity());
+  message_2_.set_optional_float(-std::numeric_limits<float>::infinity());
+  message_1_.set_optional_double(-std::numeric_limits<double>::infinity());
+  message_2_.set_optional_double(-std::numeric_limits<double>::infinity());
   comparator_.SetFractionAndMargin(field_float, 0.0, 0.0);
   comparator_.SetFractionAndMargin(field_double, 0.0, 0.0);
   EXPECT_EQ(FieldComparator::SAME,
