@@ -36,9 +36,6 @@
 #include <cstdlib>
 #include <cstring>
 #include <memory>
-#ifndef _SHARED_PTR_H
-#include <google/protobuf/stubs/shared_ptr.h>
-#endif
 
 #include <google/protobuf/stubs/logging.h>
 #include <google/protobuf/stubs/common.h>
@@ -130,7 +127,7 @@ util::Status JsonStreamParser::Parse(StringPiece json) {
     // Don't point chunk to leftover_ because leftover_ will be updated in
     // ParseChunk(chunk).
     chunk_storage_.swap(leftover_);
-    json.AppendToString(&chunk_storage_);
+    StrAppend(&chunk_storage_, json);
     chunk = StringPiece(chunk_storage_);
   }
 
@@ -141,11 +138,11 @@ util::Status JsonStreamParser::Parse(StringPiece json) {
 
     // Any leftover characters are stashed in leftover_ for later parsing when
     // there is more data available.
-    chunk.substr(n).AppendToString(&leftover_);
+    StrAppend(&leftover_, chunk.substr(n));
     return status;
   } else {
-    chunk.CopyToString(&leftover_);
-    return util::Status::OK;
+    leftover_.assign(chunk.data(), chunk.size());
+    return util::Status();
   }
 }
 
@@ -153,11 +150,11 @@ util::Status JsonStreamParser::FinishParse() {
   // If we do not expect anything and there is nothing left to parse we're all
   // done.
   if (stack_.empty() && leftover_.empty()) {
-    return util::Status::OK;
+    return util::Status();
   }
 
   // Storage for UTF8-coerced string.
-  google::protobuf::scoped_array<char> utf8;
+  std::unique_ptr<char[]> utf8;
   if (coerce_to_utf8_) {
     utf8.reset(new char[leftover_.size()]);
     char* coerced = internal::UTF8CoerceToStructurallyValid(leftover_, utf8.get(), ' ');
@@ -184,7 +181,7 @@ util::Status JsonStreamParser::FinishParse() {
 
 util::Status JsonStreamParser::ParseChunk(StringPiece chunk) {
   // Do not do any work if the chunk is empty.
-  if (chunk.empty()) return util::Status::OK;
+  if (chunk.empty()) return util::Status();
 
   p_ = json_ = chunk;
 
@@ -206,7 +203,7 @@ util::Status JsonStreamParser::ParseChunk(StringPiece chunk) {
     // unparsed data left, we save it for later parse.
     leftover_ = p_.ToString();
   }
-  return util::Status::OK;
+  return util::Status();
 }
 
 util::Status JsonStreamParser::RunParser() {
@@ -252,15 +249,15 @@ util::Status JsonStreamParser::RunParser() {
         // If we have a key we still need to render, make sure to save off the
         // contents in our own storage.
         if (!key_.empty() && key_storage_.empty()) {
-          key_.AppendToString(&key_storage_);
+          StrAppend(&key_storage_, key_);
           key_ = StringPiece(key_storage_);
         }
-        result = util::Status::OK;
+        result = util::Status();
       }
       return result;
     }
   }
-  return util::Status::OK;
+  return util::Status();
 }
 
 util::Status JsonStreamParser::ParseValue(TokenType type) {
@@ -386,7 +383,7 @@ util::Status JsonStreamParser::ParseStringHelper() {
       // start fresh.
       string_open_ = 0;
       Advance();
-      return util::Status::OK;
+      return util::Status();
     }
     // Normal character, just advance past it.
     Advance();
@@ -468,7 +465,7 @@ util::Status JsonStreamParser::ParseUnicodeEscape() {
   // Advance past the [final] code unit escape.
   p_.remove_prefix(kUnicodeEscapedLength);
   parsed_storage_.append(buf, len);
-  return util::Status::OK;
+  return util::Status();
 }
 
 util::Status JsonStreamParser::ParseNumber() {
@@ -498,6 +495,19 @@ util::Status JsonStreamParser::ParseNumber() {
   return result;
 }
 
+util::Status JsonStreamParser::ParseDoubleHelper(
+    const string& number, NumberResult* result) {
+  if (!safe_strtod(number, &result->double_val)) {
+    return ReportFailure("Unable to parse number.");
+  }
+  if (!loose_float_number_conversion_ &&
+      !MathLimits<double>::IsFinite(result->double_val)) {
+    return ReportFailure("Number exceeds the range of double.");
+  }
+  result->type = NumberResult::DOUBLE;
+  return util::Status();
+}
+
 util::Status JsonStreamParser::ParseNumberHelper(NumberResult* result) {
   const char* data = p_.data();
   int length = p_.length();
@@ -533,16 +543,11 @@ util::Status JsonStreamParser::ParseNumberHelper(NumberResult* result) {
 
   // Floating point number, parse as a double.
   if (floating) {
-    if (!safe_strtod(number, &result->double_val)) {
-      return ReportFailure("Unable to parse number.");
+    util::Status status = ParseDoubleHelper(number, result);
+    if (status.ok()) {
+      p_.remove_prefix(index);
     }
-    if (!loose_float_number_conversion_ &&
-        !MathLimits<double>::IsFinite(result->double_val)) {
-      return ReportFailure("Number exceeds the range of double.");
-    }
-    result->type = NumberResult::DOUBLE;
-    p_.remove_prefix(index);
-    return util::Status::OK;
+    return status;
   }
 
   // Positive non-floating point number, parse as a uint64.
@@ -551,12 +556,18 @@ util::Status JsonStreamParser::ParseNumberHelper(NumberResult* result) {
     if (number.length() >= 2 && number[0] == '0') {
       return ReportFailure("Octal/hex numbers are not valid JSON values.");
     }
-    if (!safe_strtou64(number, &result->uint_val)) {
-      return ReportFailure("Unable to parse number.");
+    if (safe_strtou64(number, &result->uint_val)) {
+      result->type = NumberResult::UINT;
+      p_.remove_prefix(index);
+      return util::Status();
+    } else {
+      // If the value is too large, parse it as double.
+      util::Status status = ParseDoubleHelper(number, result);
+      if (status.ok()) {
+        p_.remove_prefix(index);
+      }
+      return status;
     }
-    result->type = NumberResult::UINT;
-    p_.remove_prefix(index);
-    return util::Status::OK;
   }
 
   // Octal/Hex numbers are not valid JSON values.
@@ -564,12 +575,18 @@ util::Status JsonStreamParser::ParseNumberHelper(NumberResult* result) {
     return ReportFailure("Octal/hex numbers are not valid JSON values.");
   }
   // Negative non-floating point number, parse as an int64.
-  if (!safe_strto64(number, &result->int_val)) {
-    return ReportFailure("Unable to parse number.");
+  if (safe_strto64(number, &result->int_val)) {
+    result->type = NumberResult::INT;
+    p_.remove_prefix(index);
+    return util::Status();
+  } else {
+    // If the value is too large, parse it as double.
+    util::Status status = ParseDoubleHelper(number, result);
+    if (status.ok()) {
+      p_.remove_prefix(index);
+    }
+    return status;
   }
-  result->type = NumberResult::INT;
-  p_.remove_prefix(index);
-  return util::Status::OK;
 }
 
 util::Status JsonStreamParser::HandleBeginObject() {
@@ -578,7 +595,7 @@ util::Status JsonStreamParser::HandleBeginObject() {
   ow_->StartObject(key_);
   key_ = StringPiece();
   stack_.push(ENTRY);
-  return util::Status::OK;
+  return util::Status();
 }
 
 util::Status JsonStreamParser::ParseObjectMid(TokenType type) {
@@ -590,13 +607,13 @@ util::Status JsonStreamParser::ParseObjectMid(TokenType type) {
   if (type == END_OBJECT) {
     Advance();
     ow_->EndObject();
-    return util::Status::OK;
+    return util::Status();
   }
   // Found a comma, advance past it and get ready for an entry.
   if (type == VALUE_SEPARATOR) {
     Advance();
     stack_.push(ENTRY);
-    return util::Status::OK;
+    return util::Status();
   }
   // Illegal token after key:value pair.
   return ReportFailure("Expected , or } after key:value pair.");
@@ -611,7 +628,7 @@ util::Status JsonStreamParser::ParseEntry(TokenType type) {
   if (type == END_OBJECT) {
     ow_->EndObject();
     Advance();
-    return util::Status::OK;
+    return util::Status();
   }
 
   util::Status result;
@@ -650,7 +667,7 @@ util::Status JsonStreamParser::ParseEntryMid(TokenType type) {
   if (type == ENTRY_SEPARATOR) {
     Advance();
     stack_.push(VALUE);
-    return util::Status::OK;
+    return util::Status();
   }
   return ReportFailure("Expected : between key:value pair.");
 }
@@ -661,7 +678,7 @@ util::Status JsonStreamParser::HandleBeginArray() {
   ow_->StartList(key_);
   key_ = StringPiece();
   stack_.push(ARRAY_VALUE);
-  return util::Status::OK;
+  return util::Status();
 }
 
 util::Status JsonStreamParser::ParseArrayValue(TokenType type) {
@@ -672,7 +689,7 @@ util::Status JsonStreamParser::ParseArrayValue(TokenType type) {
   if (type == END_ARRAY) {
     ow_->EndList();
     Advance();
-    return util::Status::OK;
+    return util::Status();
   }
 
   // The ParseValue call may push something onto the stack so we need to make
@@ -696,14 +713,14 @@ util::Status JsonStreamParser::ParseArrayMid(TokenType type) {
   if (type == END_ARRAY) {
     ow_->EndList();
     Advance();
-    return util::Status::OK;
+    return util::Status();
   }
 
   // Found a comma, advance past it and expect an array value next.
   if (type == VALUE_SEPARATOR) {
     Advance();
     stack_.push(ARRAY_VALUE);
-    return util::Status::OK;
+    return util::Status();
   }
   // Illegal token after array value.
   return ReportFailure("Expected , or ] after array value.");
@@ -713,27 +730,27 @@ util::Status JsonStreamParser::ParseTrue() {
   ow_->RenderBool(key_, true);
   key_ = StringPiece();
   p_.remove_prefix(true_len);
-  return util::Status::OK;
+  return util::Status();
 }
 
 util::Status JsonStreamParser::ParseFalse() {
   ow_->RenderBool(key_, false);
   key_ = StringPiece();
   p_.remove_prefix(false_len);
-  return util::Status::OK;
+  return util::Status();
 }
 
 util::Status JsonStreamParser::ParseNull() {
   ow_->RenderNull(key_);
   key_ = StringPiece();
   p_.remove_prefix(null_len);
-  return util::Status::OK;
+  return util::Status();
 }
 
 util::Status JsonStreamParser::ParseEmptyNull() {
   ow_->RenderNull(key_);
   key_ = StringPiece();
-  return util::Status::OK;
+  return util::Status();
 }
 
 bool JsonStreamParser::IsEmptyNullAllowed(TokenType type) {
@@ -793,7 +810,7 @@ util::Status JsonStreamParser::ParseKey() {
   }
   // Since we aren't using the key storage, clear it out.
   key_storage_.clear();
-  return util::Status::OK;
+  return util::Status();
 }
 
 JsonStreamParser::TokenType JsonStreamParser::GetNextTokenType() {
