@@ -74,6 +74,8 @@ using google::protobuf::testing::Primitive;
 using google::protobuf::testing::Proto3Message;
 using google::protobuf::testing::Publisher;
 using google::protobuf::testing::StructType;
+using google::protobuf::testing::TestJsonName1;
+using google::protobuf::testing::TestJsonName2;
 using google::protobuf::testing::TimestampDuration;
 using google::protobuf::testing::ValueWrapper;
 using google::protobuf::testing::oneofs::OneOfsRequest;
@@ -144,7 +146,7 @@ class BaseProtoStreamObjectWriterTest
 
   void CheckOutput(const Message& expected, int expected_length) {
     size_t nbytes;
-    google::protobuf::scoped_array<char> buffer(output_->GetBuffer(&nbytes));
+    std::unique_ptr<char[]> buffer(output_->GetBuffer(&nbytes));
     if (expected_length >= 0) {
       EXPECT_EQ(expected_length, nbytes);
     }
@@ -152,7 +154,7 @@ class BaseProtoStreamObjectWriterTest
 
     std::stringbuf str_buf(str, std::ios_base::in);
     std::istream istream(&str_buf);
-    google::protobuf::scoped_ptr<Message> message(expected.New());
+    std::unique_ptr<Message> message(expected.New());
     message->ParsePartialFromIstream(&istream);
 
     if (!MessageDifferencer::Equivalent(expected, *message)) {
@@ -168,8 +170,8 @@ class BaseProtoStreamObjectWriterTest
 
   testing::TypeInfoTestHelper helper_;
   MockErrorListener listener_;
-  google::protobuf::scoped_ptr<GrowingArrayByteSink> output_;
-  google::protobuf::scoped_ptr<ProtoStreamObjectWriter> ow_;
+  std::unique_ptr<GrowingArrayByteSink> output_;
+  std::unique_ptr<ProtoStreamObjectWriter> ow_;
   ProtoStreamObjectWriter::Options options_;
 };
 
@@ -271,6 +273,22 @@ TEST_P(ProtoStreamObjectWriterTest, CustomJsonName) {
   CheckOutput(book);
 }
 
+// Test that two messages can have different fields mapped to the same JSON
+// name. See: https://github.com/google/protobuf/issues/1415
+TEST_P(ProtoStreamObjectWriterTest, ConflictingJsonName) {
+  ResetTypeInfo(TestJsonName1::descriptor());
+  TestJsonName1 message1;
+  message1.set_one_value(12345);
+  ow_->StartObject("")->RenderInt32("value", 12345)->EndObject();
+  CheckOutput(message1);
+
+  ResetTypeInfo(TestJsonName2::descriptor());
+  TestJsonName2 message2;
+  message2.set_another_value(12345);
+  ow_->StartObject("")->RenderInt32("value", 12345)->EndObject();
+  CheckOutput(message2);
+}
+
 TEST_P(ProtoStreamObjectWriterTest, IntEnumValuesAreAccepted) {
   Book book;
   book.set_title("Some Book");
@@ -1593,7 +1611,7 @@ TEST_P(ProtoStreamObjectWriterStructTest, RepeatedStructMapObjectKeyTest) {
 TEST_P(ProtoStreamObjectWriterStructTest, OptionStructIntAsStringsTest) {
   StructType struct_type;
   google::protobuf::Struct* s = struct_type.mutable_object();
-  s->mutable_fields()->operator[]("k1").set_number_value(123);
+  s->mutable_fields()->operator[]("k1").set_string_value("123");
   s->mutable_fields()->operator[]("k2").set_bool_value(true);
   s->mutable_fields()->operator[]("k3").set_string_value("-222222222");
   s->mutable_fields()->operator[]("k4").set_string_value("33333333");
