@@ -48,21 +48,11 @@ namespace protobuf {
 namespace util {
 namespace converter {
 
-namespace {
-const StringPiece SkipWhiteSpace(StringPiece str) {
-  StringPiece::size_type i;
-  for (i = 0; i < str.size() && isspace(str[i]); ++i) {
-  }
-  GOOGLE_DCHECK(i == str.size() || !isspace(str[i]));
-  return str.substr(i);
-}
-}  // namespace
-
 bool GetBoolOptionOrDefault(
     const google::protobuf::RepeatedPtrField<google::protobuf::Option>& options,
     const string& option_name, bool default_value) {
   const google::protobuf::Option* opt = FindOptionOrNull(options, option_name);
-  if (opt == NULL) {
+  if (opt == nullptr) {
     return default_value;
   }
   return GetBoolFromAny(opt->value());
@@ -72,7 +62,7 @@ int64 GetInt64OptionOrDefault(
     const google::protobuf::RepeatedPtrField<google::protobuf::Option>& options,
     const string& option_name, int64 default_value) {
   const google::protobuf::Option* opt = FindOptionOrNull(options, option_name);
-  if (opt == NULL) {
+  if (opt == nullptr) {
     return default_value;
   }
   return GetInt64FromAny(opt->value());
@@ -82,7 +72,7 @@ double GetDoubleOptionOrDefault(
     const google::protobuf::RepeatedPtrField<google::protobuf::Option>& options,
     const string& option_name, double default_value) {
   const google::protobuf::Option* opt = FindOptionOrNull(options, option_name);
-  if (opt == NULL) {
+  if (opt == nullptr) {
     return default_value;
   }
   return GetDoubleFromAny(opt->value());
@@ -92,7 +82,7 @@ string GetStringOptionOrDefault(
     const google::protobuf::RepeatedPtrField<google::protobuf::Option>& options,
     const string& option_name, const string& default_value) {
   const google::protobuf::Option* opt = FindOptionOrNull(options, option_name);
-  if (opt == NULL) {
+  if (opt == nullptr) {
     return default_value;
   }
   return GetStringFromAny(opt->value());
@@ -134,7 +124,10 @@ const StringPiece GetTypeWithoutUrl(StringPiece type_url) {
     return type_url.substr(kTypeUrlSize + 1);
   } else {
     size_t idx = type_url.rfind('/');
-    return type_url.substr(idx + 1);
+    if (idx != type_url.npos) {
+      type_url.remove_prefix(idx + 1);
+    }
+    return type_url;
   }
 }
 
@@ -151,12 +144,12 @@ const google::protobuf::Option* FindOptionOrNull(
       return &opt;
     }
   }
-  return NULL;
+  return nullptr;
 }
 
 const google::protobuf::Field* FindFieldInTypeOrNull(
     const google::protobuf::Type* type, StringPiece field_name) {
-  if (type != NULL) {
+  if (type != nullptr) {
     for (int i = 0; i < type->fields_size(); ++i) {
       const google::protobuf::Field& field = type->fields(i);
       if (field.name() == field_name) {
@@ -164,12 +157,12 @@ const google::protobuf::Field* FindFieldInTypeOrNull(
       }
     }
   }
-  return NULL;
+  return nullptr;
 }
 
 const google::protobuf::Field* FindJsonFieldInTypeOrNull(
     const google::protobuf::Type* type, StringPiece json_name) {
-  if (type != NULL) {
+  if (type != nullptr) {
     for (int i = 0; i < type->fields_size(); ++i) {
       const google::protobuf::Field& field = type->fields(i);
       if (field.json_name() == json_name) {
@@ -177,12 +170,12 @@ const google::protobuf::Field* FindJsonFieldInTypeOrNull(
       }
     }
   }
-  return NULL;
+  return nullptr;
 }
 
 const google::protobuf::Field* FindFieldInTypeByNumberOrNull(
     const google::protobuf::Type* type, int32 number) {
-  if (type != NULL) {
+  if (type != nullptr) {
     for (int i = 0; i < type->fields_size(); ++i) {
       const google::protobuf::Field& field = type->fields(i);
       if (field.number() == number) {
@@ -190,12 +183,12 @@ const google::protobuf::Field* FindFieldInTypeByNumberOrNull(
       }
     }
   }
-  return NULL;
+  return nullptr;
 }
 
 const google::protobuf::EnumValue* FindEnumValueByNameOrNull(
     const google::protobuf::Enum* enum_type, StringPiece enum_name) {
-  if (enum_type != NULL) {
+  if (enum_type != nullptr) {
     for (int i = 0; i < enum_type->enumvalue_size(); ++i) {
       const google::protobuf::EnumValue& enum_value = enum_type->enumvalue(i);
       if (enum_value.name() == enum_name) {
@@ -203,12 +196,12 @@ const google::protobuf::EnumValue* FindEnumValueByNameOrNull(
       }
     }
   }
-  return NULL;
+  return nullptr;
 }
 
 const google::protobuf::EnumValue* FindEnumValueByNumberOrNull(
     const google::protobuf::Enum* enum_type, int32 value) {
-  if (enum_type != NULL) {
+  if (enum_type != nullptr) {
     for (int i = 0; i < enum_type->enumvalue_size(); ++i) {
       const google::protobuf::EnumValue& enum_value = enum_type->enumvalue(i);
       if (enum_value.number() == value) {
@@ -216,12 +209,12 @@ const google::protobuf::EnumValue* FindEnumValueByNumberOrNull(
       }
     }
   }
-  return NULL;
+  return nullptr;
 }
 
 const google::protobuf::EnumValue* FindEnumValueByNameWithoutUnderscoreOrNull(
     const google::protobuf::Enum* enum_type, StringPiece enum_name) {
-  if (enum_type != NULL) {
+  if (enum_type != nullptr) {
     for (int i = 0; i < enum_type->enumvalue_size(); ++i) {
       const google::protobuf::EnumValue& enum_value = enum_type->enumvalue(i);
       string enum_name_without_underscore = enum_value.name();
@@ -242,7 +235,14 @@ const google::protobuf::EnumValue* FindEnumValueByNameWithoutUnderscoreOrNull(
       }
     }
   }
-  return NULL;
+  return nullptr;
+}
+
+string EnumValueNameToLowerCamelCase(const StringPiece input) {
+  string input_string(input);
+  std::transform(input_string.begin(), input_string.end(), input_string.begin(),
+                 ::tolower);
+  return ToCamelCase(input_string);
 }
 
 string ToCamelCase(const StringPiece input) {
@@ -359,8 +359,6 @@ bool IsMap(const google::protobuf::Field& field,
   return field.cardinality() ==
              google::protobuf::Field_Cardinality_CARDINALITY_REPEATED &&
          (GetBoolOptionOrDefault(type.options(), "map_entry", false) ||
-          GetBoolOptionOrDefault(type.options(),
-                                 "google.protobuf.MessageOptions.map_entry", false) ||
           GetBoolOptionOrDefault(type.options(),
                                  "google.protobuf.MessageOptions.map_entry",
                                  false));
@@ -369,9 +367,6 @@ bool IsMap(const google::protobuf::Field& field,
 bool IsMessageSetWireFormat(const google::protobuf::Type& type) {
   return GetBoolOptionOrDefault(type.options(), "message_set_wire_format",
                                 false) ||
-         GetBoolOptionOrDefault(type.options(),
-                                "google.protobuf.MessageOptions.message_set_wire_format",
-                                false) ||
          GetBoolOptionOrDefault(
              type.options(),
              "google.protobuf.MessageOptions.message_set_wire_format", false);
@@ -410,6 +405,13 @@ bool SafeStrToFloat(StringPiece str, float* value) {
   return true;
 }
 
+bool StringStartsWith(StringPiece text, StringPiece prefix) {
+  return text.starts_with(prefix);
+}
+
+bool StringEndsWith(StringPiece text, StringPiece suffix) {
+  return text.ends_with(suffix);
+}
 }  // namespace converter
 }  // namespace util
 }  // namespace protobuf
